package com.faraphel.tasks_valider.database.api.base

import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.database.dao.base.BaseDao
import com.faraphel.tasks_valider.database.entities.GroupEntity
import com.faraphel.tasks_valider.database.entities.base.BaseEntity
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import fi.iki.elonen.NanoHTTPD

/**
 * A base for the API to handle the database operations.
 * This is preconfigured to handle JSON data.
 * @param Entity the entity type to handle
 */
abstract class BaseJsonApi<Entity: BaseEntity> : BaseApi {
    companion object {
        private val parser = Gson()  ///< The JSON parser
    }

    /**
     * Get the DAO for the entity
     * @param database the database to get the DAO from
     * @return the DAO for the entity
     */
    abstract fun getDao(database: TaskDatabase): BaseDao<Entity>

    /**
     * Get the type token for the entity
     * @return the type token for the entity
     */
    private fun getEntityTypeToken(): TypeToken<Entity> = object: TypeToken<Entity>() {}

    // Requests

    override fun head(database: TaskDatabase, session: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        val obj = parser.fromJson<Entity>(
            session.inputStream.bufferedReader().readText(),
            getEntityTypeToken().type
        )
        val exists = this.getDao(database).exists(obj)

        return NanoHTTPD.newFixedLengthResponse(
            if (exists) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            if (exists) "Exists" else "Not found"
        )
    }

    override fun get(database: TaskDatabase, session: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.OK,
            "application/json",
            parser.toJson(database.groupDao().getAll())
        )
    }

    override fun post(database: TaskDatabase, session: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        val obj = parser.fromJson(session.inputStream.bufferedReader().readText(), GroupEntity::class.java)
        val id = database.groupDao().insert(obj)

        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.CREATED,
            "text/plain",
            id.toString()
        )
    }

    override fun delete(database: TaskDatabase, session: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        val obj = parser.fromJson(session.inputStream.bufferedReader().readText(), GroupEntity::class.java)
        val count = database.groupDao().delete(obj)

        return NanoHTTPD.newFixedLengthResponse(
            if (count > 0) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            count.toString()
        )
    }
}